/*
	Let n dogs be placed at positions (x[0], x[1], ..., x[n-1]), where the x[i] are complex<double>:s.
	Dog #i chases dog #(i+1) and runs at speed proportional to the distance to dog #(i+1) - so the chase never finishes. 
	For now, the constant of proportion is constant wrt. i.
	This program plots the chase and rescales so that the distances between dogs don't tend to zero.
	The center of mass of the dogs (which all have equal weight) happens to be located at the origin.

	Let y be x normalized.
	
	x' = (P-I)x	
	=>
	y' = <y,y>Py-<y,Py>y
	Since |y| = 1,
	y' = Py-Re(<Py,y>)y

	I is the identity matrix. P is an n x n matrix with P_{i,i+1} = 1, P_{i,j} =0 otherwise.

	---- The chase of the eigendog ----
		Let T be the matrix from init() (which is has the eigenvectors of P as its columns).
	y' = Ty-Re(<Ty,y>)y

*/
#include <iomanip>
#include <iostream>
#include <SDL/SDL.h>
#include <iostream>
#include <cassert>
#include <algorithm>
#include <climits>
#include <set>
#include <cmath>
#include <vector>
#include <cstdlib>
#include <cstdio>
#include <ctime>
#include <complex>

/*const int n_color = 6;
Uint32 color[n_color] = 
{
	SDL_MapRGB(screen->format, 255,   0,   0),
	SDL_MapRGB(screen->format,   0, 255,   0),
	SDL_MapRGB(screen->format,   0,   0, 255),
	SDL_MapRGB(screen->format, 255, 255,   0),
	SDL_MapRGB(screen->format,   0, 255, 255),
	SDL_MapRGB(screen->format, 255,   0, 255)
};*/

#define rep(i,a,b) for(int i=a;i<b;++i)
#define CR(x) const x&
#define FI(x) (x).first
#define SE(x) (x).second
#define MP(x,y) make_pair(x,y)
#define OK(x,y) (0<=(x)&&(x)<(screen->w)&&0<=(y)&&(y)<(screen->h))

using namespace std;

typedef SDL_Surface SS;
typedef float fl;
typedef pair<fl, fl> pff;
typedef pair<int,int> pii;
typedef complex<fl> cfl;
const fl EPS = 1e-9;
fl STEP = 0.001;//1.0/fl(1<<7);
const int n = 13; 
const int W = 600, H = 500;
const cfl imaginary_unit = cfl(0.0,1.0);
const cfl CONST_1 = imaginary_unit*cfl(2) *cfl(M_PI);
int cfwd[n];
cfl T[n][n];
SDL_Surface *screen;

bool dotr = false; //show y(true) or Ty(false)?
int zomer = 0;
bool sgn=true;//true=positive
fl xmin,xmax,ymin,ymax;int imin,imax,jmin,jmax;
fl rnd() {
	return fl(rand())/fl(RAND_MAX);
}
cfl e_(cfl x) {
	return exp(CONST_1*x);
}
void init() {screen=SDL_SetVideoMode(W,H, 32, SDL_SWSURFACE);
rep(i,0,n)cfwd[i]=(i+1)%n;
rep(i,0,n)rep(j,0,n)T[i][j]=e_(fl(i*j)/fl(n))/sqrt(fl(n));
fl D = 0.5;
xmin = -D, xmax = D, ymin = -D, ymax = D;
imin = 0, imax = screen->h, jmin = 0, jmax = screen->w;}
struct cvec {
	cvec();
	cfl data[n];
	CR(cfl) operator[](int k) const{assert(0<=k&&k<n);return data[k];}
	cfl& operator[](int k) {assert(0<=k&&k<n);return data[k];}
	void zero() {rep(i,0,n) data[i]=0;}
	cvec operator-() const;
};
cvec cvec::operator-() const {
	cvec rt;
	rep(i,0,n) rt[i] = -data[i];
	return rt;
}
cvec::cvec() {
	zero();
}
cvec diff(CR(cvec) y){ //returns Py - (Re<Py,y>)y
	cvec rt;
	fl k = 0;
	rep(i,0,n) k += real(y[cfwd[i]] * conj(y[i]));
	rep(i,0,n) rt[i]=y[cfwd[i]]-k*y[i];
	return rt;
}
cvec eigendiff(CR(cvec) y){ //returns Ty-(Re<Ty,y>)y
	cvec rt;
	fl k=0;
	rep(i,0,n){
		cfl a = 0;
		rep(j,0,n)
			a += T[i][j] * y[j];
		k += real(a * conj(y[i]));
	}
	rep(i,0,n){
		rt[i] = 0;
		rep(j,0,n)
			rt[i] += T[i][j]*y[j];
		rt[i] -= k * y[i];
	}
	return rt;
}
void step(cvec& state){
	cvec D = sgn?diff(state):-diff(state);
	rep(i,0,n)
		state[i] += D[i]*STEP;
}
void tow(int&i, int&j,CR(fl)x,CR(fl)y){
	i = imin + ((y-ymin)/(ymax-ymin))*fl(imax-imin); j = jmin + ((x-xmin)/(xmax-xmin))*fl(jmax-jmin);
}
void tow(int&i,int&j,CR(cfl)z){
	tow(i,j,z.real(),z.imag());
}
Uint32 getpixel(SDL_Surface *surface, int x, int y);
void putpixel(SDL_Surface *surface, int x, int y, Uint32 pixel);
void bedot(SDL_Surface *screen, int x, int y, Uint32 c,int d=2);
void line(SDL_Surface *screen,int x0,int y0,int x1,int y1,Uint32 c);
void draw(SDL_Surface *screen, cvec& state, Uint32 u){
	rep(I,0,n)	{
		int i,j,ip,jp;
		tow(i,j,state[I]),tow(ip,jp,state[cfwd[I]]);
		bedot(screen,j,i,(I==n-1||I==1)?u*256:u);
		line (screen,j,i,jp,ip,u*65536);
	}
}
void cls(SS*screen);
bool check_center(cvec& state){
	cfl s = 0;
	rep(i,0,n) s += state[i];
	if(sqrt(norm(s))<=0.01) return false;
	rep(i,0,n) state[i] -= s/fl(n);
	return true;
}
bool check_radius(cvec& state){
	fl s = 0;
	rep(i,0,n) s+=norm(state[i]);
	fl k = 1.0/sqrt(s);
	if(abs(log(k))<=0.01) return false;
	rep(i,0,n) state[i] *= k;
	return true;
}
fl dist(CR(fl) x,CR(fl)y,CR(fl)xp,CR(fl)yp){
	fl dx=x-xp,dy=y-yp;
	return sqrt(dx*dx+dy*dy);
}
int gcd(int a,int b){
	if(a < b) return gcd(b,a);
	return b?gcd(b,a%b):a;
}
int main(){
	init();
	srand(time(0));
	SDL_Init(SDL_INIT_EVERYTHING);
	SDL_Flip(screen);
	bool quit = false;
	SDL_Event e;
	cvec state;
	if(false&true)
		rep(i,0,n) state[i]=cfl(rnd()-0.5,rnd()-0.5);
	else//start with a 'paley' configuration
	{
		int t;
		while(gcd(n,t=rand()%(n))!=1);
		rep(i,0,n) state[i]=e_(fl(i*t)/fl(n))*cfl(0.1);
		cout << t << endl;
	}
	int TTL = 0;
	int TTL2 = 0;
	int TTL3 = 1<<30;
	bool hold = false;
	while(!quit)	{
		if(!hold)		{
		if(TTL < 0 &&(check_center(state) | check_radius(state)))		{
			cout << ". ", cout.flush();
			TTL = 96;//could be -1
		}
		if(TTL3<0)		{
					int x,y,i;
			//rep(I,1,2/*n/2*/)i=(zomer+2*I)%n,tow(y,x,state[2*i],state[2*i+1]),bedot(screen,x,y,255*256,2);
			}
		if(TTL2<0)		{
			if(dotr){cvec bild;
			rep(i,0,n){bild[i]=0;rep(j,0,n)bild[i] += T[i][j]*state[j];}
			draw(screen,bild,255);
			SDL_Flip(screen);
			draw(screen,bild,0);}
			else			{
				draw(screen,state,255);SDL_Flip(screen);draw(screen,state,0);
			}
			TTL2 = 50;//rand()%40;//could be -1
		}
if(TTL3<0){
			fl MM  = 0;//fl(1<<28);
			int i;
			//rep(I,1,n/2)i=(zomer+2*I)%n,MM=max(MM,dist(state[2*i],state[2*( zomer  )+1],state[2*i],state[2*i+1]));
/*		xmin = state[2*zomer]-2.0*MM;
			xmax = state[2*zomer]+2.0*MM;
			ymin = state[2*zomer+1]-2.0*MM;
			ymax = state[2*zomer+1]+2.0*MM;*/
			TTL3=0;
		}
		step(state);	
		--TTL;
		--TTL2;
		--TTL3;
		}
		while(SDL_PollEvent(&e))		{
			switch(e.type)			{
				case SDL_QUIT:
					quit=true;
					break;
				case SDL_KEYDOWN:			{
					switch(e.key.keysym.sym)					{
						case SDLK_ESCAPE:
							quit = true;
							break;
						case SDLK_SPACE:						{
							hold = true;
						}
						break;
						case SDLK_RETURN:{
							sgn ^= true;
						}
						break;
						case SDLK_j:{
							(++zomer)%=n;
						}break;
						case SDLK_f:{
							dotr ^= true;
						}break;
						case SDLK_c:{
							cls(screen);
						}
						break;
						case SDLK_t:{
							rep(T,0,1<<18) step(state);	
						}
						break;
						case SDLK_q:{STEP*=12.0;}break;case SDLK_w:{STEP*=1.20;}break;case SDLK_e:{STEP/=1.2;}break;case SDLK_r:{STEP/=12.0;}break;
					}
				}
				break;
				case SDL_KEYUP:{
					switch(e.key.keysym.sym){
						case SDLK_SPACE:{
							hold = false;
						}
						break;
					}
				}
				break;
			}
		}
	}
	SDL_Quit();
	cout<<endl;
	return 0;
}
void line(SDL_Surface *screen,int x0,int y0,int x1,int y1,Uint32 c){
	fl dx = x1-x0, dy=y1-y0;
	fl L=sqrt(dx*dx+dy*dy);
	if(L < EPS)	return;
	rep(t,0,int(L)+1){
		fl x = fl(x0)+t*dx/L, y = fl(y0)+t*dy/L;
		if(OK(x,y))	
			putpixel(screen,x,y,c);
	}
}
void bedot(SDL_Surface *screen, int x, int y, Uint32 c,int d){
	if(!(OK(x+d,y+d)&&OK(x-d,y-d)))
		return;
	for(int xpp=x-d;xpp<=x+d;++xpp)
	for(int ypp=y-d;ypp<=y+d;++ypp)
		putpixel(screen,xpp,ypp,c);
}
Uint32 getpixel(SDL_Surface *surface, int x, int y){
	int bpp = surface->format->BytesPerPixel;
	/* Here p is the address to the pixel we want to retrieve */
	Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;
	switch(bpp)	{
		case 1:
			return *p;
		case 2:
			return *(Uint16 *)p;
		case 3:
			if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
				return p[0] << 16 | p[1] << 8 | p[2];
			else
				return p[0] | p[1] << 8 | p[2] << 16;
		case 4:
			return *(Uint32 *)p;
		default:
	return 0;       /* shouldn't happen, but avoids warnings */
	}
}
void putpixel(SDL_Surface *surface, int x, int y, Uint32 pixel){
	assert(OK(x,y));
	int bpp = surface->format->BytesPerPixel;
	/* Here p is the address to the pixel we want to set */
	Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;
	switch(bpp) {
		case 1:
			*p = pixel;
			break;
		case 2:
			*(Uint16 *)p = pixel;
			break;
		case 3:
			if(SDL_BYTEORDER == SDL_BIG_ENDIAN) {
				p[0] = (pixel >> 16) & 0xff;
				p[1] = (pixel >> 8) & 0xff;
				p[2] = pixel & 0xff;
			} else {
				p[0] = pixel & 0xff;
				p[1] = (pixel >> 8) & 0xff;
				p[2] = (pixel >> 16) & 0xff;
			}
		break;
		case 4:
			*(Uint32 *)p = pixel;
		break;
	}
}
void cls(SS*s){rep(x,0,s->w)rep(y,0,s->h)putpixel(s,x,y,0);}
